/***********************
        SphereDOS
A terminal-like application in Sphere.
***********************/

var font = LoadFont("system.rfn");
var font1 = LoadFont("system.rfn");
var font2 = LoadFont("script.rfn");
var font3 = LoadFont("animeace.rfn");
var SW = GetScreenWidth();
var SH = GetScreenHeight();

var debug = true;
var rand;

var programs = "";

//Crap
var appname = "SphereDOS";
var appversion = "v1.0";
var console = new Console();

RequireScript("program.js");
RequireScript("misc.js");


function game()
{
	SetFrameRate(60);
	console.run();
}



function Console(prev)
{
	this.directory = "./";
	this.text = [""];
	this.command = "";
	this.line = 0;
	
	this.cursorspeed = 10;
	this.cursorblink = -this.cursorspeed;
	
	this.lastCommand = "";
	this.exit = false;	//True to close this console.
	
	console = this;
	
	//Reset to some defaults.
	this.reset = function()
	{
		this.text = [
			appname + " " + appversion,
			"  Copyright (c) Vincent Beers, 2007",
			"",
			this.directory + ">"
		];
		
		this.line = this.text.length-1;
		this.command = "";
	}
	
	
	//WHOA YEAH THE DOS CONSOLE
	this.run = function()
	{
		var frames = 0;
		var seconds = 0;
		var minutes = 0;
		
		var fontcount = 1;
		this.reset();
		while ((!IsKeyPressed(KEY_CTRL) || !IsKeyPressed(KEY_C)) && !IsKeyPressed(KEY_ESCAPE))
		{
			frames++;
			if (frames >= 60) frames = 0, seconds++;
			if (seconds >= 60) seconds = 0, minutes++;
			
			if (this.exit)
			{
				console = prev;
				break;
			}
			
			//console.print("Running for " + minutes + " minutes, " + seconds + " seconds and " + frames + " frames.");
			//console.print("A");
			
			//Reset when pressing F2. Debug.
			if (debug && IsKeyPressed(KEY_F2)) this.reset();
			
			//Text input
			var shift;
			while (AreKeysLeft())
			{
				//Add key to command string.
				if (IsKeyPressed(KEY_SHIFT)) shift = true;
				this.command += GetKeyString(GetKey(), shift);
				shift = false;
				
				//Remove key from command string.
				if (IsKeyPressed(KEY_BACKSPACE))
					this.command = this.command.substr(0, this.command.length-1);
				
				//Enter.
				else if (IsKeyPressed(KEY_ENTER))
					this.process(this.command);
				
				else if (IsKeyPressed(KEY_UP))
					this.command = this.lastCommand;
				
				else if (IsKeyPressed(KEY_DOWN))
					this.command = "";
			}
			
			this.scroll();	//Scrolls content IF needed.
			this.update();
		}
	}
}


//Scroll the console.
Console.prototype.scroll = function()
{
	//Scrolling.
	if ((this.text.length*font.getHeight()) > (480-font.getHeight()))
	{
		this.line--;
		for (var i = 0; i < this.text.length; i++)
			this.text[i] = this.text[i+1];
		
		this.text.pop();
	}
}


//Update the console screen.
Console.prototype.update = function()
{
	//Cursor Blink
	//If it should blink, draw it at the end of the string of the current line.
	this.cursorblink++;
	if (this.cursorblink >= 0)
	{
		font.drawText(font.getStringWidth(this.text[this.line]) + font.getStringWidth(this.command),
		this.line*font.getHeight(), "_");
	}
	if (this.cursorblink > this.cursorspeed) this.cursorblink = -this.cursorspeed-1;
	
	//Draw all lines on screen.
	for (var i = 0; i < this.text.length; i++)
	{
		if (this.text[i] == undefined) this.text[i] = "";
		font.drawText(0, i*font.getHeight(), this.text[i]);
	}
	
	font.drawText(font.getStringWidth(this.text[this.line]), this.line*font.getHeight(), this.command);
	FlipScreen();
}



//Generate output and put it on the correct lines and stuff.
Console.prototype.process = function(input)
{
	this.text[this.line] += this.command;		//Add old command to actual text.
	
	
	//RUN PROGRAMS AND COMMANDS
	switch (input.toLowerCase())
	{
		case "": this.line++; break;                            //Do nothing
		case "exit": this.exit = true; break;      //Close this instance of the console.
		
		//Open a new instance (it's nested, so you'll return to the old one when exiting).
		case appname.toLowerCase():
			this.println("Starting nested console...");
			new Console(console).run();
			this.println("Nested console exited.");
			this.line++;
			break;
		
		case "about":
			this.println(appname + " " + appversion);
			this.println("  Copyright (c) Vincent Beers, 2007");
			this.line++;
			break;
		
		case "cls":
			this.text = [];
			this.line = 0;
			break;
		
		case "shit":
		case "fuck":
		case "damn":
			this.println("Stop swearing, now.");
			this.line++;
			break;
		
		default:
			var cmd = AdvancedCommands(this, input);
			var cmdcount = 0;
			if (!cmd)
			{
				var prog = "";
				
				//Find the command till the space.
				for (var i = 0; i < input.length; i++)
				{
					if (input[i] == " ") break;
					prog += input[i];
					cmdcount++;
				}
				
				//Now check if this program exists in the programs list.
				if (programs[prog] != undefined)
				{
					var params = input.substr(cmdcount+1, input.length-cmdcount);
					programs[prog].generateParameters(params);
					programs[prog].run(params);
				}
				
				//Program is REALLY not found.
				else
					this.println("Program or command not found.");
			}
			
			this.line++;
			break;
	}
	
	//PROGRAM DONE
	this.lastCommand = this.command;
	this.command = "";		//Finally, remove command.
	this.line++;
	this.text[this.line] = this.directory + ">";
	this.scroll();
}



Console.prototype.print = function(text)
{
	this.text[this.line] += text;
}


Console.prototype.println = function(text)
{
	this.line++;
	
	//TODO? Change this so it can accept multiple line strings?
	
	this.text[this.line] = text;
	this.scroll();
}
